'use strict';

var web3_js = require('@solana/web3.js');

// env-shim.ts
var __DEV__ = /* @__PURE__ */ (() => process["env"].NODE_ENV === "development")();
async function findMarginAccountPda(seeds, config = {}) {
  const {
    programAddress = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN"
  } = config;
  return await web3_js.getProgramDerivedAddress({
    programAddress,
    seeds: [
      web3_js.getUtf8Encoder().encode("margin"),
      web3_js.getAddressEncoder().encode(seeds.tswap),
      web3_js.getAddressEncoder().encode(seeds.owner),
      web3_js.getU16Encoder().encode(seeds.marginNr)
    ]
  });
}
async function findTSwapPda(config = {}) {
  const {
    programAddress = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN"
  } = config;
  return await web3_js.getProgramDerivedAddress({ programAddress, seeds: [] });
}

// src/generated/accounts/marginAccount.ts
function getMarginAccountEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["owner", web3_js.getAddressEncoder()],
      ["name", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nr", web3_js.getU16Encoder()],
      ["bump", web3_js.getArrayEncoder(web3_js.getU8Encoder(), { size: 1 })],
      ["poolsAttached", web3_js.getU32Encoder()],
      ["reserved", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 64)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([133, 220, 173, 213, 179, 211, 43, 238])
    })
  );
}
function getMarginAccountDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["owner", web3_js.getAddressDecoder()],
    ["name", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nr", web3_js.getU16Decoder()],
    ["bump", web3_js.getArrayDecoder(web3_js.getU8Decoder(), { size: 1 })],
    ["poolsAttached", web3_js.getU32Decoder()],
    ["reserved", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 64)]
  ]);
}
function getMarginAccountCodec() {
  return web3_js.combineCodec(getMarginAccountEncoder(), getMarginAccountDecoder());
}
function decodeMarginAccount(encodedAccount) {
  return web3_js.decodeAccount(
    encodedAccount,
    getMarginAccountDecoder()
  );
}
async function fetchMarginAccount(rpc, address, config) {
  const maybeAccount = await fetchMaybeMarginAccount(rpc, address, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeMarginAccount(rpc, address, config) {
  const maybeAccount = await web3_js.fetchEncodedAccount(rpc, address, config);
  return decodeMarginAccount(maybeAccount);
}
async function fetchAllMarginAccount(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeMarginAccount(
    rpc,
    addresses,
    config
  );
  web3_js.assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeMarginAccount(rpc, addresses, config) {
  const maybeAccounts = await web3_js.fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeMarginAccount(maybeAccount));
}
function getMarginAccountSize() {
  return 143;
}
async function fetchMarginAccountFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeMarginAccountFromSeeds(
    rpc,
    seeds,
    config
  );
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeMarginAccountFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findMarginAccountPda(seeds, { programAddress });
  return await fetchMaybeMarginAccount(rpc, address, fetchConfig);
}
function getTSwapConfigEncoder() {
  return web3_js.getStructEncoder([["feeBps", web3_js.getU16Encoder()]]);
}
function getTSwapConfigDecoder() {
  return web3_js.getStructDecoder([["feeBps", web3_js.getU16Decoder()]]);
}
function getTSwapConfigCodec() {
  return web3_js.combineCodec(getTSwapConfigEncoder(), getTSwapConfigDecoder());
}

// src/generated/accounts/tSwap.ts
function getTSwapEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["version", web3_js.getU8Encoder()],
      ["bump", web3_js.getArrayEncoder(web3_js.getU8Encoder(), { size: 1 })],
      ["config", getTSwapConfigEncoder()],
      ["owner", web3_js.getAddressEncoder()],
      ["feeVault", web3_js.getAddressEncoder()],
      ["cosigner", web3_js.getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([169, 211, 171, 36, 219, 189, 79, 188])
    })
  );
}
function getTSwapDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["version", web3_js.getU8Decoder()],
    ["bump", web3_js.getArrayDecoder(web3_js.getU8Decoder(), { size: 1 })],
    ["config", getTSwapConfigDecoder()],
    ["owner", web3_js.getAddressDecoder()],
    ["feeVault", web3_js.getAddressDecoder()],
    ["cosigner", web3_js.getAddressDecoder()]
  ]);
}
function getTSwapCodec() {
  return web3_js.combineCodec(getTSwapEncoder(), getTSwapDecoder());
}
function decodeTSwap(encodedAccount) {
  return web3_js.decodeAccount(
    encodedAccount,
    getTSwapDecoder()
  );
}
async function fetchTSwap(rpc, address, config) {
  const maybeAccount = await fetchMaybeTSwap(rpc, address, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeTSwap(rpc, address, config) {
  const maybeAccount = await web3_js.fetchEncodedAccount(rpc, address, config);
  return decodeTSwap(maybeAccount);
}
async function fetchAllTSwap(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeTSwap(rpc, addresses, config);
  web3_js.assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeTSwap(rpc, addresses, config) {
  const maybeAccounts = await web3_js.fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeTSwap(maybeAccount));
}
function getTSwapSize() {
  return 108;
}
async function fetchTSwapFromSeeds(rpc, config = {}) {
  const maybeAccount = await fetchMaybeTSwapFromSeeds(rpc, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeTSwapFromSeeds(rpc, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findTSwapPda({ programAddress });
  return await fetchMaybeTSwap(rpc, address, fetchConfig);
}

// src/generated/errors/tensorEscrow.ts
var TENSOR_ESCROW_ERROR__BAD_OWNER = 6016;
var TENSOR_ESCROW_ERROR__BAD_MARGIN = 6027;
var TENSOR_ESCROW_ERROR__MARGIN_IN_USE = 6032;
var tensorEscrowErrorMessages;
if (__DEV__) {
  tensorEscrowErrorMessages = {
    [TENSOR_ESCROW_ERROR__BAD_MARGIN]: `bad margin account passed`,
    [TENSOR_ESCROW_ERROR__BAD_OWNER]: `bad owner`,
    [TENSOR_ESCROW_ERROR__MARGIN_IN_USE]: `margin account has pools open and is in use`
  };
}
function getTensorEscrowErrorMessage(code) {
  if (__DEV__) {
    return tensorEscrowErrorMessages[code];
  }
  return "Error message not available in production bundles. Compile with `__DEV__` set to true to see more information.";
}
var TENSOR_ESCROW_PROGRAM_ADDRESS = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
var TensorEscrowAccount = /* @__PURE__ */ ((TensorEscrowAccount2) => {
  TensorEscrowAccount2[TensorEscrowAccount2["MarginAccount"] = 0] = "MarginAccount";
  TensorEscrowAccount2[TensorEscrowAccount2["TSwap"] = 1] = "TSwap";
  return TensorEscrowAccount2;
})(TensorEscrowAccount || {});
function identifyTensorEscrowAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([133, 220, 173, 213, 179, 211, 43, 238])
    ),
    0
  )) {
    return 0 /* MarginAccount */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([169, 211, 171, 36, 219, 189, 79, 188])
    ),
    0
  )) {
    return 1 /* TSwap */;
  }
  throw new Error(
    "The provided account could not be identified as a tensorEscrow account."
  );
}
var TensorEscrowInstruction = /* @__PURE__ */ ((TensorEscrowInstruction2) => {
  TensorEscrowInstruction2[TensorEscrowInstruction2["InitUpdateTswap"] = 0] = "InitUpdateTswap";
  TensorEscrowInstruction2[TensorEscrowInstruction2["InitMarginAccount"] = 1] = "InitMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["CloseMarginAccount"] = 2] = "CloseMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["DepositMarginAccount"] = 3] = "DepositMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["WithdrawMarginAccount"] = 4] = "WithdrawMarginAccount";
  return TensorEscrowInstruction2;
})(TensorEscrowInstruction || {});
function identifyTensorEscrowInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([140, 185, 54, 172, 15, 94, 31, 155])
    ),
    0
  )) {
    return 0 /* InitUpdateTswap */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([10, 54, 68, 252, 130, 97, 39, 52])
    ),
    0
  )) {
    return 1 /* InitMarginAccount */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([105, 215, 41, 239, 166, 207, 1, 103])
    ),
    0
  )) {
    return 2 /* CloseMarginAccount */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([190, 85, 242, 60, 119, 81, 33, 192])
    ),
    0
  )) {
    return 3 /* DepositMarginAccount */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([54, 73, 150, 208, 207, 5, 18, 17])
    ),
    0
  )) {
    return 4 /* WithdrawMarginAccount */;
  }
  throw new Error(
    "The provided instruction could not be identified as a tensorEscrow instruction."
  );
}
function expectSome(value) {
  if (value == null) {
    throw new Error("Expected a value but received null or undefined.");
  }
  return value;
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: web3_js.AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? web3_js.AccountRole.WRITABLE : web3_js.AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? web3_js.upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && web3_js.isTransactionSigner(value);
}

// src/generated/instructions/closeMarginAccount.ts
function getCloseMarginAccountInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([105, 215, 41, 239, 166, 207, 1, 103])
    })
  );
}
function getCloseMarginAccountInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)]
  ]);
}
function getCloseMarginAccountInstructionDataCodec() {
  return web3_js.combineCodec(
    getCloseMarginAccountInstructionDataEncoder(),
    getCloseMarginAccountInstructionDataDecoder()
  );
}
async function getCloseMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCloseMarginAccountInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCloseMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCloseMarginAccountInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCloseMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getDepositMarginAccountInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["lamports", web3_js.getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([190, 85, 242, 60, 119, 81, 33, 192])
    })
  );
}
function getDepositMarginAccountInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["lamports", web3_js.getU64Decoder()]
  ]);
}
function getDepositMarginAccountInstructionDataCodec() {
  return web3_js.combineCodec(
    getDepositMarginAccountInstructionDataEncoder(),
    getDepositMarginAccountInstructionDataDecoder()
  );
}
async function getDepositMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: 0
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDepositMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getDepositMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDepositMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDepositMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDepositMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getInitMarginAccountInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["marginNr", web3_js.getU16Encoder()],
      ["name", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([10, 54, 68, 252, 130, 97, 39, 52]),
      marginNr: value.marginNr ?? 0,
      name: value.name ?? new Uint8Array([
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0
      ])
    })
  );
}
function getInitMarginAccountInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["marginNr", web3_js.getU16Decoder()],
    ["name", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)]
  ]);
}
function getInitMarginAccountInstructionDataCodec() {
  return web3_js.combineCodec(
    getInitMarginAccountInstructionDataEncoder(),
    getInitMarginAccountInstructionDataDecoder()
  );
}
async function getInitMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!args.marginNr) {
    args.marginNr = 0;
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: expectSome(args.marginNr)
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getInitMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getInitMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!args.marginNr) {
    args.marginNr = 0;
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getInitMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInitMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getInitMarginAccountInstructionDataDecoder().decode(instruction.data)
  };
}
function getInitUpdateTswapInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["config", getTSwapConfigEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([140, 185, 54, 172, 15, 94, 31, 155])
    })
  );
}
function getInitUpdateTswapInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["config", getTSwapConfigDecoder()]
  ]);
}
function getInitUpdateTswapInstructionDataCodec() {
  return web3_js.combineCodec(
    getInitUpdateTswapInstructionDataEncoder(),
    getInitUpdateTswapInstructionDataDecoder()
  );
}
async function getInitUpdateTswapInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    newOwner: { value: input.newOwner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.newOwner)
    ],
    programAddress,
    data: getInitUpdateTswapInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getInitUpdateTswapInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    newOwner: { value: input.newOwner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.newOwner)
    ],
    programAddress,
    data: getInitUpdateTswapInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInitUpdateTswapInstruction(instruction) {
  if (instruction.accounts.length < 6) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      feeVault: getNextAccount(),
      cosigner: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount(),
      newOwner: getNextAccount()
    },
    data: getInitUpdateTswapInstructionDataDecoder().decode(instruction.data)
  };
}
function getWithdrawMarginAccountInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["lamports", web3_js.getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([54, 73, 150, 208, 207, 5, 18, 17])
    })
  );
}
function getWithdrawMarginAccountInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["lamports", web3_js.getU64Decoder()]
  ]);
}
function getWithdrawMarginAccountInstructionDataCodec() {
  return web3_js.combineCodec(
    getWithdrawMarginAccountInstructionDataEncoder(),
    getWithdrawMarginAccountInstructionDataDecoder()
  );
}
async function getWithdrawMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: 0
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getWithdrawMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getWithdrawMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getWithdrawMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseWithdrawMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getWithdrawMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}

exports.TENSOR_ESCROW_ERROR__BAD_MARGIN = TENSOR_ESCROW_ERROR__BAD_MARGIN;
exports.TENSOR_ESCROW_ERROR__BAD_OWNER = TENSOR_ESCROW_ERROR__BAD_OWNER;
exports.TENSOR_ESCROW_ERROR__MARGIN_IN_USE = TENSOR_ESCROW_ERROR__MARGIN_IN_USE;
exports.TENSOR_ESCROW_PROGRAM_ADDRESS = TENSOR_ESCROW_PROGRAM_ADDRESS;
exports.TensorEscrowAccount = TensorEscrowAccount;
exports.TensorEscrowInstruction = TensorEscrowInstruction;
exports.decodeMarginAccount = decodeMarginAccount;
exports.decodeTSwap = decodeTSwap;
exports.fetchAllMarginAccount = fetchAllMarginAccount;
exports.fetchAllMaybeMarginAccount = fetchAllMaybeMarginAccount;
exports.fetchAllMaybeTSwap = fetchAllMaybeTSwap;
exports.fetchAllTSwap = fetchAllTSwap;
exports.fetchMarginAccount = fetchMarginAccount;
exports.fetchMarginAccountFromSeeds = fetchMarginAccountFromSeeds;
exports.fetchMaybeMarginAccount = fetchMaybeMarginAccount;
exports.fetchMaybeMarginAccountFromSeeds = fetchMaybeMarginAccountFromSeeds;
exports.fetchMaybeTSwap = fetchMaybeTSwap;
exports.fetchMaybeTSwapFromSeeds = fetchMaybeTSwapFromSeeds;
exports.fetchTSwap = fetchTSwap;
exports.fetchTSwapFromSeeds = fetchTSwapFromSeeds;
exports.findMarginAccountPda = findMarginAccountPda;
exports.findTSwapPda = findTSwapPda;
exports.getCloseMarginAccountInstruction = getCloseMarginAccountInstruction;
exports.getCloseMarginAccountInstructionAsync = getCloseMarginAccountInstructionAsync;
exports.getCloseMarginAccountInstructionDataCodec = getCloseMarginAccountInstructionDataCodec;
exports.getCloseMarginAccountInstructionDataDecoder = getCloseMarginAccountInstructionDataDecoder;
exports.getCloseMarginAccountInstructionDataEncoder = getCloseMarginAccountInstructionDataEncoder;
exports.getDepositMarginAccountInstruction = getDepositMarginAccountInstruction;
exports.getDepositMarginAccountInstructionAsync = getDepositMarginAccountInstructionAsync;
exports.getDepositMarginAccountInstructionDataCodec = getDepositMarginAccountInstructionDataCodec;
exports.getDepositMarginAccountInstructionDataDecoder = getDepositMarginAccountInstructionDataDecoder;
exports.getDepositMarginAccountInstructionDataEncoder = getDepositMarginAccountInstructionDataEncoder;
exports.getInitMarginAccountInstruction = getInitMarginAccountInstruction;
exports.getInitMarginAccountInstructionAsync = getInitMarginAccountInstructionAsync;
exports.getInitMarginAccountInstructionDataCodec = getInitMarginAccountInstructionDataCodec;
exports.getInitMarginAccountInstructionDataDecoder = getInitMarginAccountInstructionDataDecoder;
exports.getInitMarginAccountInstructionDataEncoder = getInitMarginAccountInstructionDataEncoder;
exports.getInitUpdateTswapInstruction = getInitUpdateTswapInstruction;
exports.getInitUpdateTswapInstructionAsync = getInitUpdateTswapInstructionAsync;
exports.getInitUpdateTswapInstructionDataCodec = getInitUpdateTswapInstructionDataCodec;
exports.getInitUpdateTswapInstructionDataDecoder = getInitUpdateTswapInstructionDataDecoder;
exports.getInitUpdateTswapInstructionDataEncoder = getInitUpdateTswapInstructionDataEncoder;
exports.getMarginAccountCodec = getMarginAccountCodec;
exports.getMarginAccountDecoder = getMarginAccountDecoder;
exports.getMarginAccountEncoder = getMarginAccountEncoder;
exports.getMarginAccountSize = getMarginAccountSize;
exports.getTSwapCodec = getTSwapCodec;
exports.getTSwapConfigCodec = getTSwapConfigCodec;
exports.getTSwapConfigDecoder = getTSwapConfigDecoder;
exports.getTSwapConfigEncoder = getTSwapConfigEncoder;
exports.getTSwapDecoder = getTSwapDecoder;
exports.getTSwapEncoder = getTSwapEncoder;
exports.getTSwapSize = getTSwapSize;
exports.getTensorEscrowErrorMessage = getTensorEscrowErrorMessage;
exports.getWithdrawMarginAccountInstruction = getWithdrawMarginAccountInstruction;
exports.getWithdrawMarginAccountInstructionAsync = getWithdrawMarginAccountInstructionAsync;
exports.getWithdrawMarginAccountInstructionDataCodec = getWithdrawMarginAccountInstructionDataCodec;
exports.getWithdrawMarginAccountInstructionDataDecoder = getWithdrawMarginAccountInstructionDataDecoder;
exports.getWithdrawMarginAccountInstructionDataEncoder = getWithdrawMarginAccountInstructionDataEncoder;
exports.identifyTensorEscrowAccount = identifyTensorEscrowAccount;
exports.identifyTensorEscrowInstruction = identifyTensorEscrowInstruction;
exports.parseCloseMarginAccountInstruction = parseCloseMarginAccountInstruction;
exports.parseDepositMarginAccountInstruction = parseDepositMarginAccountInstruction;
exports.parseInitMarginAccountInstruction = parseInitMarginAccountInstruction;
exports.parseInitUpdateTswapInstruction = parseInitUpdateTswapInstruction;
exports.parseWithdrawMarginAccountInstruction = parseWithdrawMarginAccountInstruction;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.js.map