import { getProgramDerivedAddress, getUtf8Encoder, getAddressEncoder, getU16Encoder, transformEncoder, getStructEncoder, fixEncoderSize, getBytesEncoder, getArrayEncoder, getU8Encoder, getU32Encoder, getStructDecoder, fixDecoderSize, getBytesDecoder, getAddressDecoder, getU16Decoder, getArrayDecoder, getU8Decoder, getU32Decoder, combineCodec, decodeAccount, assertAccountExists, fetchEncodedAccount, assertAccountsExist, fetchEncodedAccounts, containsBytes, getU64Encoder, getU64Decoder, AccountRole, upgradeRoleToSigner, isTransactionSigner as isTransactionSigner$1 } from '@solana/web3.js';

// env-shim.ts
var __DEV__ = /* @__PURE__ */ (() => process["env"].NODE_ENV === "development")();
async function findMarginAccountPda(seeds, config = {}) {
  const {
    programAddress = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN"
  } = config;
  return await getProgramDerivedAddress({
    programAddress,
    seeds: [
      getUtf8Encoder().encode("margin"),
      getAddressEncoder().encode(seeds.tswap),
      getAddressEncoder().encode(seeds.owner),
      getU16Encoder().encode(seeds.marginNr)
    ]
  });
}
async function findTSwapPda(config = {}) {
  const {
    programAddress = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN"
  } = config;
  return await getProgramDerivedAddress({ programAddress, seeds: [] });
}

// src/generated/accounts/marginAccount.ts
function getMarginAccountEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["owner", getAddressEncoder()],
      ["name", fixEncoderSize(getBytesEncoder(), 32)],
      ["nr", getU16Encoder()],
      ["bump", getArrayEncoder(getU8Encoder(), { size: 1 })],
      ["poolsAttached", getU32Encoder()],
      ["reserved", fixEncoderSize(getBytesEncoder(), 64)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([133, 220, 173, 213, 179, 211, 43, 238])
    })
  );
}
function getMarginAccountDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["owner", getAddressDecoder()],
    ["name", fixDecoderSize(getBytesDecoder(), 32)],
    ["nr", getU16Decoder()],
    ["bump", getArrayDecoder(getU8Decoder(), { size: 1 })],
    ["poolsAttached", getU32Decoder()],
    ["reserved", fixDecoderSize(getBytesDecoder(), 64)]
  ]);
}
function getMarginAccountCodec() {
  return combineCodec(getMarginAccountEncoder(), getMarginAccountDecoder());
}
function decodeMarginAccount(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getMarginAccountDecoder()
  );
}
async function fetchMarginAccount(rpc, address, config) {
  const maybeAccount = await fetchMaybeMarginAccount(rpc, address, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeMarginAccount(rpc, address, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address, config);
  return decodeMarginAccount(maybeAccount);
}
async function fetchAllMarginAccount(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeMarginAccount(
    rpc,
    addresses,
    config
  );
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeMarginAccount(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeMarginAccount(maybeAccount));
}
function getMarginAccountSize() {
  return 143;
}
async function fetchMarginAccountFromSeeds(rpc, seeds, config = {}) {
  const maybeAccount = await fetchMaybeMarginAccountFromSeeds(
    rpc,
    seeds,
    config
  );
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeMarginAccountFromSeeds(rpc, seeds, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findMarginAccountPda(seeds, { programAddress });
  return await fetchMaybeMarginAccount(rpc, address, fetchConfig);
}
function getTSwapConfigEncoder() {
  return getStructEncoder([["feeBps", getU16Encoder()]]);
}
function getTSwapConfigDecoder() {
  return getStructDecoder([["feeBps", getU16Decoder()]]);
}
function getTSwapConfigCodec() {
  return combineCodec(getTSwapConfigEncoder(), getTSwapConfigDecoder());
}

// src/generated/accounts/tSwap.ts
function getTSwapEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["version", getU8Encoder()],
      ["bump", getArrayEncoder(getU8Encoder(), { size: 1 })],
      ["config", getTSwapConfigEncoder()],
      ["owner", getAddressEncoder()],
      ["feeVault", getAddressEncoder()],
      ["cosigner", getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([169, 211, 171, 36, 219, 189, 79, 188])
    })
  );
}
function getTSwapDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["version", getU8Decoder()],
    ["bump", getArrayDecoder(getU8Decoder(), { size: 1 })],
    ["config", getTSwapConfigDecoder()],
    ["owner", getAddressDecoder()],
    ["feeVault", getAddressDecoder()],
    ["cosigner", getAddressDecoder()]
  ]);
}
function getTSwapCodec() {
  return combineCodec(getTSwapEncoder(), getTSwapDecoder());
}
function decodeTSwap(encodedAccount) {
  return decodeAccount(
    encodedAccount,
    getTSwapDecoder()
  );
}
async function fetchTSwap(rpc, address, config) {
  const maybeAccount = await fetchMaybeTSwap(rpc, address, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeTSwap(rpc, address, config) {
  const maybeAccount = await fetchEncodedAccount(rpc, address, config);
  return decodeTSwap(maybeAccount);
}
async function fetchAllTSwap(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeTSwap(rpc, addresses, config);
  assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeTSwap(rpc, addresses, config) {
  const maybeAccounts = await fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeTSwap(maybeAccount));
}
function getTSwapSize() {
  return 108;
}
async function fetchTSwapFromSeeds(rpc, config = {}) {
  const maybeAccount = await fetchMaybeTSwapFromSeeds(rpc, config);
  assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeTSwapFromSeeds(rpc, config = {}) {
  const { programAddress, ...fetchConfig } = config;
  const [address] = await findTSwapPda({ programAddress });
  return await fetchMaybeTSwap(rpc, address, fetchConfig);
}

// src/generated/errors/tensorEscrow.ts
var TENSOR_ESCROW_ERROR__BAD_OWNER = 6016;
var TENSOR_ESCROW_ERROR__BAD_MARGIN = 6027;
var TENSOR_ESCROW_ERROR__MARGIN_IN_USE = 6032;
var tensorEscrowErrorMessages;
if (__DEV__) {
  tensorEscrowErrorMessages = {
    [TENSOR_ESCROW_ERROR__BAD_MARGIN]: `bad margin account passed`,
    [TENSOR_ESCROW_ERROR__BAD_OWNER]: `bad owner`,
    [TENSOR_ESCROW_ERROR__MARGIN_IN_USE]: `margin account has pools open and is in use`
  };
}
function getTensorEscrowErrorMessage(code) {
  if (__DEV__) {
    return tensorEscrowErrorMessages[code];
  }
  return "Error message not available in production bundles. Compile with `__DEV__` set to true to see more information.";
}
var TENSOR_ESCROW_PROGRAM_ADDRESS = "TSWAPaqyCSx2KABk68Shruf4rp7CxcNi8hAsbdwmHbN";
var TensorEscrowAccount = /* @__PURE__ */ ((TensorEscrowAccount2) => {
  TensorEscrowAccount2[TensorEscrowAccount2["MarginAccount"] = 0] = "MarginAccount";
  TensorEscrowAccount2[TensorEscrowAccount2["TSwap"] = 1] = "TSwap";
  return TensorEscrowAccount2;
})(TensorEscrowAccount || {});
function identifyTensorEscrowAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([133, 220, 173, 213, 179, 211, 43, 238])
    ),
    0
  )) {
    return 0 /* MarginAccount */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([169, 211, 171, 36, 219, 189, 79, 188])
    ),
    0
  )) {
    return 1 /* TSwap */;
  }
  throw new Error(
    "The provided account could not be identified as a tensorEscrow account."
  );
}
var TensorEscrowInstruction = /* @__PURE__ */ ((TensorEscrowInstruction2) => {
  TensorEscrowInstruction2[TensorEscrowInstruction2["InitUpdateTswap"] = 0] = "InitUpdateTswap";
  TensorEscrowInstruction2[TensorEscrowInstruction2["InitMarginAccount"] = 1] = "InitMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["CloseMarginAccount"] = 2] = "CloseMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["DepositMarginAccount"] = 3] = "DepositMarginAccount";
  TensorEscrowInstruction2[TensorEscrowInstruction2["WithdrawMarginAccount"] = 4] = "WithdrawMarginAccount";
  return TensorEscrowInstruction2;
})(TensorEscrowInstruction || {});
function identifyTensorEscrowInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([140, 185, 54, 172, 15, 94, 31, 155])
    ),
    0
  )) {
    return 0 /* InitUpdateTswap */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([10, 54, 68, 252, 130, 97, 39, 52])
    ),
    0
  )) {
    return 1 /* InitMarginAccount */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([105, 215, 41, 239, 166, 207, 1, 103])
    ),
    0
  )) {
    return 2 /* CloseMarginAccount */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([190, 85, 242, 60, 119, 81, 33, 192])
    ),
    0
  )) {
    return 3 /* DepositMarginAccount */;
  }
  if (containsBytes(
    data,
    fixEncoderSize(getBytesEncoder(), 8).encode(
      new Uint8Array([54, 73, 150, 208, 207, 5, 18, 17])
    ),
    0
  )) {
    return 4 /* WithdrawMarginAccount */;
  }
  throw new Error(
    "The provided instruction could not be identified as a tensorEscrow instruction."
  );
}
function expectSome(value) {
  if (value == null) {
    throw new Error("Expected a value but received null or undefined.");
  }
  return value;
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? AccountRole.WRITABLE : AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && isTransactionSigner$1(value);
}

// src/generated/instructions/closeMarginAccount.ts
function getCloseMarginAccountInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([["discriminator", fixEncoderSize(getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([105, 215, 41, 239, 166, 207, 1, 103])
    })
  );
}
function getCloseMarginAccountInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)]
  ]);
}
function getCloseMarginAccountInstructionDataCodec() {
  return combineCodec(
    getCloseMarginAccountInstructionDataEncoder(),
    getCloseMarginAccountInstructionDataDecoder()
  );
}
async function getCloseMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCloseMarginAccountInstructionDataEncoder().encode({})
  };
  return instruction;
}
function getCloseMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCloseMarginAccountInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCloseMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getDepositMarginAccountInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["lamports", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([190, 85, 242, 60, 119, 81, 33, 192])
    })
  );
}
function getDepositMarginAccountInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["lamports", getU64Decoder()]
  ]);
}
function getDepositMarginAccountInstructionDataCodec() {
  return combineCodec(
    getDepositMarginAccountInstructionDataEncoder(),
    getDepositMarginAccountInstructionDataDecoder()
  );
}
async function getDepositMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: 0
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDepositMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getDepositMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDepositMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDepositMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDepositMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getInitMarginAccountInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["marginNr", getU16Encoder()],
      ["name", fixEncoderSize(getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([10, 54, 68, 252, 130, 97, 39, 52]),
      marginNr: value.marginNr ?? 0,
      name: value.name ?? new Uint8Array([
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0
      ])
    })
  );
}
function getInitMarginAccountInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["marginNr", getU16Decoder()],
    ["name", fixDecoderSize(getBytesDecoder(), 32)]
  ]);
}
function getInitMarginAccountInstructionDataCodec() {
  return combineCodec(
    getInitMarginAccountInstructionDataEncoder(),
    getInitMarginAccountInstructionDataDecoder()
  );
}
async function getInitMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!args.marginNr) {
    args.marginNr = 0;
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: expectSome(args.marginNr)
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getInitMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getInitMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!args.marginNr) {
    args.marginNr = 0;
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getInitMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInitMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getInitMarginAccountInstructionDataDecoder().decode(instruction.data)
  };
}
function getInitUpdateTswapInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["config", getTSwapConfigEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([140, 185, 54, 172, 15, 94, 31, 155])
    })
  );
}
function getInitUpdateTswapInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["config", getTSwapConfigDecoder()]
  ]);
}
function getInitUpdateTswapInstructionDataCodec() {
  return combineCodec(
    getInitUpdateTswapInstructionDataEncoder(),
    getInitUpdateTswapInstructionDataDecoder()
  );
}
async function getInitUpdateTswapInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    newOwner: { value: input.newOwner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.newOwner)
    ],
    programAddress,
    data: getInitUpdateTswapInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getInitUpdateTswapInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: true },
    feeVault: { value: input.feeVault ?? null, isWritable: false },
    cosigner: { value: input.cosigner ?? null, isWritable: false },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    newOwner: { value: input.newOwner ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.feeVault),
      getAccountMeta(accounts.cosigner),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.newOwner)
    ],
    programAddress,
    data: getInitUpdateTswapInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInitUpdateTswapInstruction(instruction) {
  if (instruction.accounts.length < 6) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      feeVault: getNextAccount(),
      cosigner: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount(),
      newOwner: getNextAccount()
    },
    data: getInitUpdateTswapInstructionDataDecoder().decode(instruction.data)
  };
}
function getWithdrawMarginAccountInstructionDataEncoder() {
  return transformEncoder(
    getStructEncoder([
      ["discriminator", fixEncoderSize(getBytesEncoder(), 8)],
      ["lamports", getU64Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([54, 73, 150, 208, 207, 5, 18, 17])
    })
  );
}
function getWithdrawMarginAccountInstructionDataDecoder() {
  return getStructDecoder([
    ["discriminator", fixDecoderSize(getBytesDecoder(), 8)],
    ["lamports", getU64Decoder()]
  ]);
}
function getWithdrawMarginAccountInstructionDataCodec() {
  return combineCodec(
    getWithdrawMarginAccountInstructionDataEncoder(),
    getWithdrawMarginAccountInstructionDataDecoder()
  );
}
async function getWithdrawMarginAccountInstructionAsync(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tswap.value) {
    accounts.tswap.value = await findTSwapPda();
  }
  if (!accounts.marginAccount.value) {
    accounts.marginAccount.value = await findMarginAccountPda({
      tswap: expectAddress(accounts.tswap.value),
      owner: expectAddress(accounts.owner.value),
      marginNr: 0
    });
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getWithdrawMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function getWithdrawMarginAccountInstruction(input) {
  const programAddress = TENSOR_ESCROW_PROGRAM_ADDRESS;
  const originalAccounts = {
    tswap: { value: input.tswap ?? null, isWritable: false },
    marginAccount: { value: input.marginAccount ?? null, isWritable: true },
    owner: { value: input.owner ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.tswap),
      getAccountMeta(accounts.marginAccount),
      getAccountMeta(accounts.owner),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getWithdrawMarginAccountInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseWithdrawMarginAccountInstruction(instruction) {
  if (instruction.accounts.length < 4) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      tswap: getNextAccount(),
      marginAccount: getNextAccount(),
      owner: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getWithdrawMarginAccountInstructionDataDecoder().decode(
      instruction.data
    )
  };
}

export { TENSOR_ESCROW_ERROR__BAD_MARGIN, TENSOR_ESCROW_ERROR__BAD_OWNER, TENSOR_ESCROW_ERROR__MARGIN_IN_USE, TENSOR_ESCROW_PROGRAM_ADDRESS, TensorEscrowAccount, TensorEscrowInstruction, decodeMarginAccount, decodeTSwap, fetchAllMarginAccount, fetchAllMaybeMarginAccount, fetchAllMaybeTSwap, fetchAllTSwap, fetchMarginAccount, fetchMarginAccountFromSeeds, fetchMaybeMarginAccount, fetchMaybeMarginAccountFromSeeds, fetchMaybeTSwap, fetchMaybeTSwapFromSeeds, fetchTSwap, fetchTSwapFromSeeds, findMarginAccountPda, findTSwapPda, getCloseMarginAccountInstruction, getCloseMarginAccountInstructionAsync, getCloseMarginAccountInstructionDataCodec, getCloseMarginAccountInstructionDataDecoder, getCloseMarginAccountInstructionDataEncoder, getDepositMarginAccountInstruction, getDepositMarginAccountInstructionAsync, getDepositMarginAccountInstructionDataCodec, getDepositMarginAccountInstructionDataDecoder, getDepositMarginAccountInstructionDataEncoder, getInitMarginAccountInstruction, getInitMarginAccountInstructionAsync, getInitMarginAccountInstructionDataCodec, getInitMarginAccountInstructionDataDecoder, getInitMarginAccountInstructionDataEncoder, getInitUpdateTswapInstruction, getInitUpdateTswapInstructionAsync, getInitUpdateTswapInstructionDataCodec, getInitUpdateTswapInstructionDataDecoder, getInitUpdateTswapInstructionDataEncoder, getMarginAccountCodec, getMarginAccountDecoder, getMarginAccountEncoder, getMarginAccountSize, getTSwapCodec, getTSwapConfigCodec, getTSwapConfigDecoder, getTSwapConfigEncoder, getTSwapDecoder, getTSwapEncoder, getTSwapSize, getTensorEscrowErrorMessage, getWithdrawMarginAccountInstruction, getWithdrawMarginAccountInstructionAsync, getWithdrawMarginAccountInstructionDataCodec, getWithdrawMarginAccountInstructionDataDecoder, getWithdrawMarginAccountInstructionDataEncoder, identifyTensorEscrowAccount, identifyTensorEscrowInstruction, parseCloseMarginAccountInstruction, parseDepositMarginAccountInstruction, parseInitMarginAccountInstruction, parseInitUpdateTswapInstruction, parseWithdrawMarginAccountInstruction };
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.mjs.map